@@ -26,15 +26,19 @@
 
 #include <ctype.h>
 #include <regex.h>
+#include <math.h> /* for lroundf */
 #include <limits.h>
 
 extern pfmon_support_t pfmon_itanium;
 extern pfmon_support_t pfmon_itanium2;
 extern pfmon_support_t pfmon_montecito;
 extern pfmon_support_t pfmon_generic_ia64;
-extern pfmon_support_t pfmon_amd_x86_64;
+extern pfmon_support_t pfmon_amd64;
+extern pfmon_support_t pfmon_i386_pm;
 extern pfmon_support_t pfmon_i386_p6;
+extern pfmon_support_t pfmon_pentium4;
 extern pfmon_support_t pfmon_gen_ia32;
+extern pfmon_support_t pfmon_coreduo;
 
 static pfmon_support_t *pfmon_cpus[]={
 #ifdef CONFIG_PFMON_ITANIUM
@@ -49,13 +53,19 @@ static pfmon_support_t *pfmon_cpus[]={
 #ifdef CONFIG_PFMON_GEN_IA64
 	&pfmon_generic_ia64,		/* must always be last of IA-64 choices */
 #endif
-#ifdef CONFIG_PFMON_AMD_X86_64
-	&pfmon_amd_x86_64,
+#ifdef CONFIG_PFMON_AMD64
+	&pfmon_amd64,
 #endif
 #ifdef CONFIG_PFMON_I386_P6
 	&pfmon_i386_p6,
+	&pfmon_i386_pm,
+#endif
+
+#ifdef CONFIG_PFMON_PENTIUM4
+	&pfmon_pentium4,
 #endif
 #ifdef CONFIG_PFMON_GEN_IA32
+	&pfmon_coreduo,
 	&pfmon_gen_ia32,	/* must always be last of IA-32 choices */
 #endif
 	NULL
@@ -243,8 +253,7 @@ prepare_pmd_smpl(pfmon_event_set_t *set)
 static int
 prepare_pmd_registers(pfmon_event_set_t *set)
 {
-	unsigned int i, j;
-	unsigned int reg_num;
+	unsigned int i;
 
 	/*
 	 * install initial value, long and short overflow rates
@@ -252,35 +261,18 @@ prepare_pmd_registers(pfmon_event_set_t *set)
 	 * Mapping from PMC -> PMD:
 	 * The logic is that each user-specified event corresponds to one counter (PMD).
 	 * On some PMU models, it is necessary to configure several PMC registers per PMD.
-	 * Libpfm guarantees:
-	 * 	- pfp_pmcs[] provides PMC infos in the same order as input events it received
-	 * 	- PMC configuring the same PMD are contiguous in pfp_pmcs[]
 	 */
-	for(i=0, j=0; i < set->event_count; i++) {
-		/*
-		 * extract the PMD idx that corresponds to event i
-		 */
-		if (pfmon_current->pfmon_pmd_map && pfmon_current->pfmon_pmd_map(set->outp.pfp_pmcs[j].reg_pmd_num, &reg_num)) {
-			fatal_error("cannot find mapping for libpfm pmd%u\n", set->outp.pfp_pmcs[j].reg_pmd_num);
-		} else {
-			reg_num = set->outp.pfp_pmcs[j].reg_pmd_num;
-		}
-		set->master_pd[i].reg_num	  = reg_num;
-
+	for(i=0; i < set->event_count; i++) {
+		set->master_pd[i].reg_num	  = set->outp.pfp_pmds[i].reg_num;
 		set->master_pd[i].reg_set         = set->id;
 		set->master_pd[i].reg_value       = set->long_rates[i].value;
 		set->master_pd[i].reg_short_reset = set->short_rates[i].value;
 		set->master_pd[i].reg_long_reset  = set->long_rates[i].value;
 		set->master_pd[i].reg_random_mask = set->long_rates[i].mask;	/* mask is per monitor */
 		set->master_pd[i].reg_random_seed = set->long_rates[i].seed;	/* seed is per monitor */
-		set->tmp_pd[i].reg_num            = reg_num;
+		set->tmp_pd[i].reg_num            = set->outp.pfp_pmds[i].reg_num;
 		set->tmp_pd[i].reg_set            = set->id;
 
-		/*
-		 * skip identical PMD, other PMC for the same PMD
-		 */
-		for(; j < set->outp.pfp_pmc_count; j++)  if (set->outp.pfp_pmcs[j].reg_evt_idx != i) break;
-
 		vbprintf("[pmd%u set=%u ival=0x%"PRIx64
 			 " long_rate=0x%"PRIx64
 			 " short_rate=0x%"PRIx64
@@ -364,9 +356,6 @@ install_event_sets(pfmon_sdesc_t *sdesc)
 			if (sdesc->nsets > 1)
 				sd.set_flags |= PFM_SETFL_TIME_SWITCH;
 
-			if (set->set_flags & PFM_SETFL_EXCL_IDLE)
-				vbprintf("CPU%d set%u excluding idle thread\n", sdesc->cpu, set->id);
-
 			if (pfmon_create_evtsets(ctxid, &sd, 1) == -1) {
 				warning("cannot create event sets: %s\n", strerror(errno));
 				return -1;
@@ -661,7 +650,7 @@ static struct option pfmon_common_options[]={
 	{ "trigger-data-stop-addresses", 1, 0, 32},
 	//{ "interval", 2, 0, 33},
 	{ "switch-timeout", 1, 0, 35},
-	{ "excl-idle", 0, 0, 36},
+	//{ "excl-idle", 0, 0, 36},
 	{ "extra-smpl-pmds", 1, 0, 37},
 
 	{ "verbose", 0, &options.opt_verbose, 1 },
@@ -714,78 +703,131 @@ usage(char **argv)
 {
 	printf("usage: %s [OPTIONS]... COMMAND\n", argv[0]);
 
-	printf(	"-h, --help\t\t\t\tdisplay this help and exit\n"
-		"-V, --version\t\t\t\toutput version information and exit\n"
-		"-l[regex], --show-events[=regex]\tdisplay all or a matching subset of the events\n"
-		"--long-show-events[=regex]\t\tdisplay all or a matching subset of the events with info\n"
-		"-i event, --event-info=event\t\tdisplay information about an event (numeric code or regex)\n"
-		"-u, -3 --user-level\t\t\tmonitor at the user level for all events (default: on)\n"
-		"-k, -0 --kernel-level\t\t\tmonitor at the kernel level for all events (default: off)\n"
-		"-1\t\t\t\t\tmonitor execution at privilege level 1 (default: off)\n"
-		"-2\t\t\t\t\tmonitor execution at privilege level 2 (default: off)\n"
-		"-e, --events=ev1,ev2,...\t\tselect events to monitor\n"
-		"-I,--info\t\t\t\tlist supported PMU models and compiled in sampling output formats\n"
-		"-t secs, --session-timeout=secs\t\tduration of the system wide session in seconds\n"
-		"-S format, --smpl-module-info=format\tdisplay information about a sampling output format\n"
-		"--debug\t\t\t\t\tenable debug prints\n"
-		"--verbose\t\t\t\tprint more information during execution\n"
-		"--outfile=filename\t\t\tprint results in a file\n"
-		"--append\t\t\t\tappend results to outfile\n"
-		"--overflow-block\t\t\tblock the task when sampling buffer is full (default: off)\n"
-		"--system-wide\t\t\t\tcreate a system wide monitoring session (default: per-task)\n"
-		"--smpl-outfile=filename\t\t\tfile to save the sampling results\n"
-		"--long-smpl-periods=val1,val2,...\tset sampling period after user notification\n"
-		"--short-smpl-periods=val1,val2,...\tset sampling period\n"
-		"--smpl-entries=n\t\t\tnumber of entries in sampling buffer\n"
-		"--with-header\t\t\t\tgenerate a header for results\n"
-		"--cpu-list=num,num1-num2,...\t\tspecify list, via numbers, of CPUs for system-wide session (default: all)\n"
-		"--aggregate-results\t\t\taggregate counts and sampling buffer outputs for multi CPU monitoring (default: off)\n"
-		"--trigger-code-start-address=addr\tstart monitoring only when code address is executed\n"
-		"--trigger-code-stop-address=addr\tstop monitoring when code address is executed\n"
-		"--trigger-data-start-address=addr\tstart monitoring only when data address is accessed\n"
-		"--trigger-data-stop-address=addr\tstop monitoring when data address code is accessed\n"
-		"--trigger-code-repeat\t\t\tstart/stop monitoring each time trigger start/stop are executed\n"
-		"--trigger-code-follow\t\t\tstart/stop code trigger applied to all monitored task (default first only)\n"
-		"--trigger-data-repeat\t\t\tstart/stop monitoring each time trigger start/stop are accessed\n"
-		"--trigger-data-follow\t\t\tstart/stop data trigger applied to all monitored task (default first only)\n"
-		"--trigger-data-ro\t\t\tdata trigger activated on read access (default read-write)\n"
-		"--trigger-data-wo\t\t\tdata trigger activated on write access (default read-write)\n"
-		"--trigger-start-delay=secs\t\tnumber of seconds before activating monitoring\n"
-		"--priv-levels=lvl1,lvl2,...\t\tset privilege level per event (any combination of [0123uk]))\n"
-		"--us-counter-format\t\t\tprint counters using commas (e.g., 1,024)\n"
-		"--eu-counter-format\t\t\tprint counters using points (e.g., 1.024)\n"
-		"--hex-counter-format\t\t\tprint counters in hexadecimal (e.g., 0x400)\n"
-		"--smpl-module=name\t\t\tselect sampling module, use -I to list modules\n"
-		"--show-time\t\t\t\tshow real,user, and system time for the command executed\n"
-		"--symbol-file=filename\t\t\tELF image containing a symbol table\n"
-		"--sysmap-file=filename\t\t\tSystem.map-format file containing a symbol table\n"
-		"--check-events-only\t\t\tverify combination of events and exit (no measurement)\n"
-		"--smpl-periods-random=mask1:seed1,...\tapply randomization to long and short periods\n"
-		"--smpl-print-counts\t\t\tprint counters values when sampling session ends (default: no)\n"
-		"--attach-task pid\t\t\tmonitor process identified by pid\n"
-		"--reset-non-smpl-periods\t\ton overflow reset counters not used as sampling periods\n"
-		"--follow-fork\t\t\t\tmonitoring continues across fork\n"
-		"--follow-vfork\t\t\t\tmonitoring continues across vfork\n"
-		"--follow-pthread\t\t\tmonitoring continues across pthread_create\n"
-		"--follow-exec[=pattern]\t\t\tfollow exec with optional command pattern\n"
-		"--follow-exec-exclude=pattern\t\tfollow exec but exclude commands matching the pattern\n"
-		"--follow-all\t\t\t\tfollow fork, vfork, exec, pthreads\n"
-		"--no-cmd-output\t\t\t\tredirect all output of executed commands to /dev/null\n"
-		"--exec-split-results\t\t\tgenerate separate results output on execve()\n"
-		"--resolve-addresses\t\t\ttry to resolve code/data addresses to symbols\n"
-		"--extra-smpl-pmds=num,num1-num2,...\tspecify a list of extra PMD register to include in samples\n"
+	/*                1         2         3         4         5         6         7         8  */
+	/*       12345678901234567890123456789012345678901234567890123456789012345678901234567890  */
+	printf(	"-h, --help\t\t\t\tDisplay this help and exit.\n"
+		"-V, --version\t\t\t\tOutput version information and exit.\n"
+		"-l[regex], --show-events[=regex]\tDisplay all or a matching subset of\n"
+		"\t\t\t\t\t the events.\n"
+		"--long-show-events[=regex]\t\tDisplay all or a matching subset of\n"
+		"\t\t\t\t\t the events with info.\n"
+		"-i event, --event-info=event\t\tDisplay information about an event\n"
+		"\t\t\t\t\t (numeric code or regex).\n"
+		"-u, -3 --user-level\t\t\tMonitor at the user level for all\n"
+		"\t\t\t\t\t events (default: on).\n"
+		"-k, -0 --kernel-level\t\t\tMonitor at the kernel level for all\n"
+		"\t\t\t\t\t events (default: off).\n"
+		"-1\t\t\t\t\tMonitor execution at privilege level 1\n"
+		"\t\t\t\t\t (default: off).\n"
+		"-2\t\t\t\t\tMonitor execution at privilege level 2\n"
+		"\t\t\t\t\t (default: off).\n"
+		"-e, --events=ev1,ev2,...\t\tSelect events to monitor.\n"
+		"-I,--info\t\t\t\tList supported PMU models and compiled\n"
+		"\t\t\t\t\t in sampling output formats.\n"
+		"-t secs, --session-timeout=secs\t\tDuration of the system wide session in\n"
+		"\t\t\t\t\t seconds.\n"
+		"-S format, --smpl-module-info=format\tDisplay information about a sampling\n"
+		"\t\t\t\t\t output format.\n"
+		"--debug\t\t\t\t\tEnable debug prints.\n"
+		"--verbose\t\t\t\tPrint more information during execution.\n"
+		"--outfile=filename\t\t\tPrint results in a file.\n"
+		"--append\t\t\t\tAppend results to outfile.\n"
+		"--overflow-block\t\t\tBlock the task when sampling buffer is\n"
+		"\t\t\t\t\t full (default: off).\n"
+		"--system-wide\t\t\t\tCreate a system wide monitoring session\n"
+		"\t\t\t\t\t (default: per-task).\n"
+		"--smpl-outfile=filename\t\t\tFile to save the sampling results.\n"
+		"--long-smpl-periods=val1,val2,...\tSet sampling period after user\n"
+		"\t\t\t\t\t notification.\n"
+		"--short-smpl-periods=val1,val2,...\tSet sampling period.\n"
+		"--smpl-entries=n\t\t\tNumber of entries in sampling buffer.\n"
+		"--with-header\t\t\t\tGenerate a header for results.\n"
+		"--cpu-list=num,num1-num2,...\t\tSpecify list, via numbers, of CPUs for\n"
+		"\t\t\t\t\t system-wide session (default: all).\n"
+		"--aggregate-results\t\t\tAggregate counts and sampling buffer\n"
+		"\t\t\t\t\t outputs for multi CPU monitoring\n"
+		"\t\t\t\t\t (default: off).\n"
+		"--trigger-code-start-address=addr\tStart monitoring only when code address\n"
+		"\t\t\t\t\t is executed.\n"
+		"--trigger-code-stop-address=addr\tStop monitoring when code address is\n"
+		"\t\t\t\t\t executed.\n"
+		"--trigger-data-start-address=addr\tStart monitoring only when data address\n"
+		"\t\t\t\t\t is accessed.\n"
+		"--trigger-data-stop-address=addr\tStop monitoring when data address code\n"
+		"\t\t\t\t\t is accessed.\n"
+		"--trigger-code-repeat\t\t\tStart/stop monitoring each time trigger\n"
+		"\t\t\t\t\t start/stop are executed.\n"
+		"--trigger-code-follow\t\t\tStart/stop code trigger applied to all\n"
+		"\t\t\t\t\t monitored task (default first only).\n"
+		"--trigger-data-repeat\t\t\tStart/stop monitoring each time trigger\n"
+		"\t\t\t\t\t start/stop are accessed.\n"
+		"--trigger-data-follow\t\t\tStart/stop data trigger applied to all\n"
+		"\t\t\t\t\t monitored task (default first only).\n"
+		"--trigger-data-ro\t\t\tData trigger activated on read access\n"
+		"\t\t\t\t\t (default read-write).\n"
+		"--trigger-data-wo\t\t\tData trigger activated on write access\n"
+		"\t\t\t\t\t (default read-write).\n"
+		"--trigger-start-delay=secs\t\tNumber of seconds before activating\n"
+		"\t\t\t\t\t monitoring.\n"
+		"--priv-levels=lvl1,lvl2,...\t\tSet privilege level per event (any\n"
+		"\t\t\t\t\t combination of [0123uk]).\n"
+		"--us-counter-format\t\t\tPrint counters using commas\n"
+		"\t\t\t\t\t (e.g., 1,024).\n"
+		"--eu-counter-format\t\t\tPrint counters using points\n"
+		"\t\t\t\t\t (e.g., 1.024).\n"
+		"--hex-counter-format\t\t\tPrint counters in hexadecimal\n"
+		"\t\t\t\t\t (e.g., 0x400).\n"
+		"--smpl-module=name\t\t\tSelect sampling module, use -I to list\n"
+		"\t\t\t\t\t modules.\n"
+		"--show-time\t\t\t\tShow real,user, and system time for the\n"
+		"\t\t\t\t\t command executed.\n"
+		"--symbol-file=filename\t\t\tELF image containing a symbol table.\n"
+		"--sysmap-file=filename\t\t\tSystem.map-format file containing a\n"
+		"\t\t\t\t\t symbol table.\n"
+		"--check-events-only\t\t\tVerify combination of events and exit\n"
+		"\t\t\t\t\t (no measurement).\n"
+		"--smpl-periods-random=mask1:seed1,...\tApply randomization to long and short\n"
+		"\t\t\t\t\t periods.\n"
+		"--smpl-print-counts\t\t\tPrint counters values when sampling\n"
+		"\t\t\t\t\t session ends (default: no).\n"
+		"--attach-task pid\t\t\tMonitor process identified by pid.\n"
+		"--reset-non-smpl-periods\t\tOn overflow reset counters not used as\n"
+		"\t\t\t\t\t sampling periods.\n"
+		"--follow-fork\t\t\t\tMonitoring continues across fork.\n"
+		"--follow-vfork\t\t\t\tMonitoring continues across vfork.\n"
+		"--follow-pthread\t\t\tMonitoring continues across\n"
+		"\t\t\t\t\t pthread_create.\n"
+		"--follow-exec[=pattern]\t\t\tFollow exec with optional command\n"
+		"\t\t\t\t\t pattern.\n"
+		"--follow-exec-exclude=pattern\t\tFollow exec but exclude commands\n"
+		"\t\t\t\t\t matching the pattern.\n"
+		"--follow-all\t\t\t\tFollow fork, vfork, exec, pthreads.\n"
+		"--no-cmd-output\t\t\t\tRedirect all output of executed commands\n"
+		"\t\t\t\t\t to /dev/null.\n"
+		"--exec-split-results\t\t\tGenerate separate results output on\n"
+		"\t\t\t\t\t execve().\n"
+		"--resolve-addresses\t\t\tTry to resolve code/data addresses to\n"
+		"\t\t\t\t\t symbols.\n"
+		"--extra-smpl-pmds=num,num1-num2,...\tSpecify a list of extra PMD register to\n"
+		"\t\t\t\t\t include in samples.\n"
 
 #ifdef CONFIG_PFMON_DEMANGLE
-		"--demangle-cpp\t\t\t\tC++ symbol demangling (default: off)\n"
-		"--demangle-java\t\t\t\tJava symbol demangling (default: off)\n"
+		"--demangle-cpp\t\t\t\tC++ symbol demangling (default: off).\n"
+		"--demangle-java\t\t\t\tJava symbol demangling (default: off).\n"
 #endif
-		"--saturate-smpl-buffer\t\t\tonly collect samples until buffer becomes full\n"
-		"--interval[=n]\t\t\t\tnumber of seconds to multiplex all event sets (default 1s)\n"
-		"--pin-command\t\t\t\tpin executed command on --cpu-list (system-wide only)\n"
-		"--switch-timeout=microseconds\t\tthe number of microseconds before switching to the next event set\n"
-		"--dont-start\t\t\t\tdo not activate monitoring in pfmon (per-thread only)\n"
-		"--excl-idle\t\t\t\texclude idle task from system-wide measurement\n"
-		"--cpu-set-relative\t\t\tCPU identifications relative to cpu_set affinity (default: off)\n"
+		"--saturate-smpl-buffer\t\t\tOnly collect samples until buffer\n"
+		"\t\t\t\t\t becomes full.\n"
+		"--interval[=n]\t\t\t\tNumber of seconds to multiplex all event\n"
+		"\t\t\t\t\t sets (default 1s).\n"
+		"--pin-command\t\t\t\tPin executed command on --cpu-list\n"
+		"\t\t\t\t\t (system-wide only).\n"
+		"--switch-timeout=microseconds\t\tThe number of microseconds before\n"
+		"\t\t\t\t\t switching to the next event set.\n"
+		"--dont-start\t\t\t\tDo not activate monitoring in pfmon\n"
+		"\t\t\t\t\t (per-thread only).\n"
+		"\t\t\t\t\t measurement.\n"
+		"--cpu-set-relative\t\t\tCPU identifications relative to cpu_set\n"
+		"\t\t\t\t\t affinity (default: off).\n"
 	);
 
 	if (pfmon_current->pfmon_usage) pfmon_current->pfmon_usage();
@@ -933,6 +975,10 @@ setup_trigger_addresses(void)
 	count = 0;
 
 	if (options.code_trigger_start) {
+
+		if (options.opt_code_trigger_repeat && options.code_trigger_stop == 0)
+			fatal_error("cannot use --trigger-code-repeat without --trigger-code-stop & --trigger-code-start\n");
+
 		parse_trigger_list(options.code_trigger_start, PFMON_TEXT_SYMBOL, 1, trg1, max, &count);
 		/*
 		 * we have some triggered start, therefore we do not start right away
@@ -945,6 +991,10 @@ setup_trigger_addresses(void)
 	count = 0;
 
 	if (options.code_trigger_stop) {
+
+		if (options.opt_code_trigger_repeat && options.code_trigger_start == 0)
+			fatal_error("cannot use --trigger-code-repeat without --trigger-code-stop & --trigger-code-start\n");
+
 		parse_trigger_list(options.code_trigger_stop, PFMON_TEXT_SYMBOL, 0, trg1, max, &count);
 	}
 	trg1 += count;
@@ -960,6 +1010,10 @@ setup_trigger_addresses(void)
 	count = 0;
 
 	if (options.data_trigger_start) {
+
+		if (options.opt_data_trigger_repeat && options.data_trigger_stop == 0)
+			fatal_error("cannot use --trigger-data-repeat without --trigger-data-stop & --trigger-data-start\n");
+
 		parse_trigger_list(options.data_trigger_start, PFMON_DATA_SYMBOL, 1, trg1, max, &count);
 		/*
 		 * we have some triggered start, therefore we do not start right away
@@ -972,6 +1026,10 @@ setup_trigger_addresses(void)
 	count = 0;
 
 	if (options.data_trigger_stop) {
+
+		if (options.opt_data_trigger_repeat && options.data_trigger_start == 0)
+			fatal_error("cannot use --trigger-data-repeat without --trigger-data-stop & --trigger-data-start\n");
+
 		parse_trigger_list(options.data_trigger_stop, PFMON_DATA_SYMBOL, 0, trg1, max, &count);
 	}
 	trg1 += count;
@@ -1047,33 +1105,90 @@ setup_trigger_addresses(void)
 	}
 }
 
+static void
+__pfmon_show_event_info(unsigned int idx)
+{
+	pfmlib_regmask_t cnt, impl_cnt;
+	char *desc;
+	unsigned int n, i, c;
+	int code;
+	char name[PFMON_MAX_EVTNAME_LEN];
+
+	pfm_get_event_name(idx, name, PFMON_MAX_EVTNAME_LEN);
+	pfm_get_event_code(idx, &code);
+	pfm_get_event_counters(idx, &cnt);
+	pfm_get_num_counters(&n);
+	pfm_get_impl_counters(&impl_cnt);
+
+	printf("Name     : %s\n"
+			"Code     : 0x%x\n"
+			"Counters : [ "
+			,
+			name,
+			code);
+
+	for (i=0; n; i++) {
+		if (pfm_regmask_isset(&impl_cnt, i))
+			n--;
+		if (pfm_regmask_isset(&cnt, i))
+			printf("%d ", i);
+	}
+	puts("]");
+
+	pfm_get_num_event_masks(idx, &n);
+	pfm_get_event_description(idx, &desc);
+ 	printf("Desc     : %s\n", desc);
+	free(desc);
+	for (i = 0; n; n--, i++) {
+		pfm_get_event_mask_description(idx, i, &desc);
+		pfm_get_event_mask_name(idx, i, name, PFMON_MAX_EVTNAME_LEN);
+		pfm_get_event_mask_code(idx, i, &c);
+		printf("Umask-%02u : 0x%02x : [%s] : %s\n", i, c, name, desc);
+		free(desc);
+	}
+
+	if (pfmon_current->pfmon_show_event_info)
+		pfmon_current->pfmon_show_event_info(idx);
+
+}
+
 static int
-pfmon_print_event_info(char *event)
+pfmon_show_event_info(char *event)
 {
 	regex_t preg;
-	int done = PFMLIB_ERR_NOTFOUND;
-	unsigned int i, count;
+	int found = 0;
+	unsigned int i, c;
+	int code, ret;
+	unsigned long lcode;
 	char name[PFMON_MAX_EVTNAME_LEN];
 
 	if (isdigit(*event)) {
-		done = pfm_print_event_info(event, printf);
-		goto skip_regex;
+
+		/* using strotul() allows base auto-detection */
+		lcode = strtoul(event, NULL, 0);
+		if (lcode >= INT_MAX)
+			fatal_error("invalid code %s\n", lcode);
+		code = (int) lcode;
+
+		ret = pfm_find_event_bycode(code, &c);
+		if (ret != PFMLIB_SUCCESS)
+			fatal_error("no event with code 0x%x\n", code);
+		__pfmon_show_event_info(c);
+		return 0;
 	}
 
-	if (regcomp(&preg, event, REG_ICASE|REG_NOSUB)) {
+	if (regcomp(&preg, event, REG_ICASE|REG_NOSUB))
 		fatal_error("error in regular expression for event \"%s\"\n", event);
-	}
 
-	pfm_get_num_events(&count);
-	for(i=0; i < count; i++) {
+	pfm_get_num_events(&c);
+	for(i=0; i < c; i++) {
 		pfm_get_event_name(i, name, PFMON_MAX_EVTNAME_LEN);
 		if (regexec(&preg, name, 0, NULL, 0) == 0) {
-			pfm_print_event_info_byindex(i, printf);
-			done = PFMLIB_SUCCESS;
+			__pfmon_show_event_info(i);
+			found = 1;
 		}
 	}
-skip_regex:
-	if (done != PFMLIB_SUCCESS)
+	if (!found)
 		fatal_error("event \"%s\" not found\n", event);
 
 	return 0;
@@ -1082,11 +1197,13 @@ skip_regex:
 static void
 pfmon_show_info(void)
 {
-	unsigned int version;
+	unsigned int version, num_cnt;
 	pfmon_support_t **supp;
 
+	pfm_get_num_counters(&num_cnt);
 	pfmon_print_simple_cpuinfo(stdout, "detected host CPUs: ");
 	printf("detected PMU model: %s\n", pfmon_current->name ? pfmon_current->name : "None");
+	printf("max counters/set: %u\n", num_cnt);
 	printf("supported PMU models: ");
 	for(supp = pfmon_cpus; *supp; supp++) {
 		printf("[%s] ", (*supp)->name);
@@ -1121,9 +1238,10 @@ static void
 pfmon_initialize(char **argv)
 {
 	long long_val;
+	pfmlib_event_t e;
+	struct timespec ts;
 	char *str;
 	size_t len;
-	unsigned int idx;
 	int ret;
 #define VERSION_22 (2U<<16|2U)
 
@@ -1132,11 +1250,8 @@ pfmon_initialize(char **argv)
 	/*
 	 * check if kernel supports event sets
 	 */
-	if (options.pfm_version >= VERSION_22) {
+	if (options.pfm_version >= VERSION_22)
 		options.opt_is22 = 1;
-		if (pfmon_check_kernel_pmu())
-			fatal_error("The kernel does not have any PMU description registered\n");
-	}
 
 	pfmon_collect_affinity();
 
@@ -1184,8 +1299,9 @@ pfmon_initialize(char **argv)
 
 	options.config_cpus = long_val;
 
+	clock_getres(CLOCK_REALTIME, &ts);
 
-	long_val = sysconf(_SC_CLK_TCK);
+	long_val = lroundf(1000000000.0 / (double)ts.tv_nsec);
 	if (long_val == -1) 
 		fatal_error("cannot figure out the clock tick\n");
 
@@ -1193,6 +1309,13 @@ pfmon_initialize(char **argv)
 	options.page_size   = getpagesize();
 	options.cpu_mhz     = pfmon_find_cpu_speed();
 
+	pfm_get_max_event_name_len(&len);
+	options.ev_name1 = malloc(len+len+1+1);
+	if (!options.ev_name1)
+		fatal_error("cannot allocate temporary event name buffers\n");
+
+	options.ev_name2 = options.ev_name1+len+1;
+
 	/*
 	 * invoke model-specific initialization, if any
 	 * (register model-specific options)
@@ -1215,10 +1338,11 @@ pfmon_initialize(char **argv)
 	 */
 	pfmon_smpl_initialize();
 
-	pfm_get_cycle_event(&idx);
-	pfm_get_max_event_name_len(&len);
+	pfm_get_cycle_event(&e);
 	str = malloc(len+1);
-	pfm_get_event_name(idx, str, len);	
+	if (!str)
+		fatal_error("cannot create default set\n");
+	pfm_get_full_event_name(&e, str, len+1);	
 	/*
 	 * create a default event set
 	 */
@@ -1524,17 +1648,9 @@ pfmon_verify_cmdline_options(int argc, char **argv)
 void
 pfmon_verify_event_sets(void)
 {
-	pfmon_event_set_t *set;
-
 	if (options.nsets > 1 && options.opt_is22 == 0) 
 		fatal_error("kernel has no support for event sets, you cannot have more than one set\n");
 
-	if (options.opt_syst_wide == 0) {
-		for(set = options.sets; set; set = set->next) {
-			if (set->set_flags & PFM_SETFL_EXCL_IDLE) 
-				fatal_error("excl-idle is not available for per-task measurement\n");
-		}
-	}
 	/*
 	 * does not return in case of error
 	 */
@@ -1586,7 +1702,7 @@ main(int argc, char **argv)
 
 			case   1:
 			case 'i':
-				exit(pfmon_print_event_info(optarg));
+				exit(pfmon_show_event_info(optarg));
 
 			case   2:
 			case 'l':
@@ -1877,18 +1993,6 @@ main(int argc, char **argv)
 
 				switch_timeout = long_val;
 				break;
-
-			case 36 :
-				if (options.last_set == NULL)
-					fatal_error("you need to define an event set first\n");
-
-				if (options.opt_is22 == 0)
-					fatal_error("--excl-idle is not supported by your host"
-						    "kernel perfmon subsystem. You need at least perfmon v2.2\n");
-
-				options.last_set->set_flags |= PFM_SETFL_EXCL_IDLE;
-				break;
-
 			case 37 :
 				if (options.last_set == NULL)
 					fatal_error("you need to define an event set first\n");
