@@ -43,12 +43,16 @@
 #endif
 
 #ifdef __x86_64__
-#include "pfmon_amd_x86_64.h"
+#include "pfmon_amd64.h"
 #endif
 
 #ifdef __i386__
 #include "pfmon_i386.h"
-#include "pfmon_amd_x86_64.h"
+#include "pfmon_amd64.h"
+#endif
+
+#ifdef __mips__
+#include "pfmon_mips64.h"
 #endif
 
 typedef pfarg_pmd_t	pfmon_pmd_t;
@@ -469,6 +473,8 @@ typedef struct {
 	pfmon_event_set_t	*last_set;	/* points to active set in the list */
 	unsigned int		nsets;		/* number of sets in the list */
 	size_t			max_event_name_len;	/* max len for event name (pretty printing) */
+	char			*ev_name1;	/* can hold full event name (cannot share) */
+	char			*ev_name2;	/* can hold full event name (cannot share) */
 	pfmon_smpl_module_t	*smpl_mod;			/* which sampling module to use */
 	size_t			ctx_arg_size;
 	void 			*model_options;			/* point to model specific pfmon options */
@@ -538,7 +544,8 @@ typedef struct {
 	int	(*pfmon_setup_ctx_flags)(pfmon_ctx_t *ctx);
 	void	(*pfmon_verify_event_sets)(void);
 	void    (*pfmon_verify_cmdline)(int argc, char **argv);
-	void	(*pfmon_detailed_event_name)(int evt);
+	void	(*pfmon_detailed_event_name)(unsigned int evt);
+	void	(*pfmon_show_event_info)(unsigned int evt);
 	char	*name;			/* support module name */
 	int	pmu_type;		/* indicate the PMU type, must be one from pfmlib.h */
 	pfmon_pmu_mask_t generic_pmu_type;	/* what is the generic PMU type for the architecture */
@@ -597,12 +604,12 @@ extern int pfmon_detect_unavail_pmcs(pfmlib_regmask_t *r_pmcs);
 
 
 /* helper functions provided by arch-specific code */
-extern int  pfmon_set_code_breakpoint(pid_t pid, int dbreg, unsigned long address);
-extern int  pfmon_clear_code_breakpoint(pid_t pid, int dbreg, unsigned long address);
-extern int  pfmon_set_data_breakpoint(pid_t pid, int dbreg, unsigned long address, int rw);
-extern int  pfmon_clear_data_breakpoint(pid_t pid, int dbreg, unsigned long address);
-extern int  pfmon_resume_after_code_breakpoint(pid_t pid);
-extern int  pfmon_resume_after_data_breakpoint(pid_t pid);
+extern int  pfmon_set_code_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start);
+extern int  pfmon_clear_code_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start);
+extern int  pfmon_set_data_breakpoint(pid_t pid, int dbreg, unsigned long address, int rw, int is_start);
+extern int  pfmon_clear_data_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start);
+extern int  pfmon_resume_after_code_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start);
+extern int  pfmon_resume_after_data_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start);
 extern void pfmon_arch_initialize(void);
 extern int  pfmon_enable_all_breakpoints(pid_t pid);
 extern int  pfmon_disable_all_breakpoints(pid_t pid);
