@@ -276,30 +276,38 @@ pfmon_ita_install_pmd_registers(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 static void
 pfmon_ita_usage(void)
 {
-	printf(
-		"--event-thresholds=thr1,thr2,...\tset event thresholds (no space)\n"
-		"--opc-match8=[mifb]:match:mask\t\tset opcode match for pmc8\n"
-		"--opc-match9=[mifb]:match:mask\t\tset opcode match for pmc9\n"
-		"--btb-no-tar\t\t\t\tdon't capture TAR predictions\n"
-		"--btb-no-bac\t\t\t\tdon't capture BAC predictions\n"
-		"--btb-no-tac\t\t\t\tdon't capture TAC predictions\n"
-		"--btb-tm-tk\t\t\t\tcapture taken IA-64 branches only\n"
-		"--btb-tm-ntk\t\t\t\tcapture not taken IA-64 branches only\n"
-		"--btb-ptm-correct\t\t\tcapture branch if target predicted correctly\n"
-		"--btb-ptm-incorrect\t\t\tcapture branch if target is mispredicted\n"
-		"--btb-ppm-correct\t\t\tcapture branch if path is predicted correctly\n"
-		"--btb-ppm-incorrect\t\t\tcapture branch if path is mispredicted\n"
-		"--btb-all-mispredicted\t\t\tcapture all mispredicted branches\n"
-		"--irange=start-end\t\t\tspecify an instruction address range constraint\n"
-		"--drange=start-end\t\t\tspecify a data address range constraint\n"
-		"--checkpoint-func=addr\t\t\ta bundle address to use as checkpoint\n"
-		"--ia32\t\t\t\t\tmonitor IA-32 execution only\n"
-		"--ia64\t\t\t\t\tmonitor IA-64 execution only\n"
-		"--insn-sets=set1,set2,...\t\tset per event instruction set (setX=[ia32|ia64|both])\n"
-		"--no-qual-check\t\t\t\tdo not check qualifier constraints on events\n"
-		"--insecure\t\t\t\tallow rum/sum in monitored task(per-thread mode and root only)\n"
-		"--excl-intr\t\t\t\texclude interrupt-triggered execution from system-wide measurement\n"
-		"--intr-only\t\t\t\tinclude only interrupt-triggered execution from system-wide measurement\n"
+	printf( "--event-thresholds=thr1,thr2,...\tSet event thresholds (no space).\n"
+		"--opc-match8=[mifb]:match:mask\t\tSet opcode match for pmc8.\n"
+		"--opc-match9=[mifb]:match:mask\t\tSet opcode match for pmc9.\n"
+		"--btb-no-tar\t\t\t\tDon't capture TAR predictions.\n"
+		"--btb-no-bac\t\t\t\tDon't capture BAC predictions.\n"
+		"--btb-no-tac\t\t\t\tDon't capture TAC predictions.\n"
+		"--btb-tm-tk\t\t\t\tCapture taken IA-64 branches only.\n"
+		"--btb-tm-ntk\t\t\t\tCapture not taken IA-64 branches only.\n"
+		"--btb-ptm-correct\t\t\tCapture branch if target predicted\n"
+		"\t\t\t\t\t correctly.\n"
+		"--btb-ptm-incorrect\t\t\tCapture branch if target is\n"
+		"\t\t\t\t\t mispredicted.\n"
+		"--btb-ppm-correct\t\t\tCapture branch if path is predicted\n"
+		"\t\t\t\t\t correctly.\n"
+		"--btb-ppm-incorrect\t\t\tCapture branch if path is mispredicted.\n"
+		"--btb-all-mispredicted\t\t\tCapture all mispredicted branches.\n"
+		"--irange=start-end\t\t\tSpecify an instruction address range\n"
+		"\t\t\t\t\t constraint.\n"
+		"--drange=start-end\t\t\tSpecify a data address range constraint.\n"
+		"--checkpoint-func=addr\t\t\tA bundle address to use as checkpoint.\n"
+		"--ia32\t\t\t\t\tMonitor IA-32 execution only.\n"
+		"--ia64\t\t\t\t\tMonitor IA-64 execution only.\n"
+		"--insn-sets=set1,set2,...\t\tSet per event instruction set\n"
+		"\t\t\t\t\t (setX=[ia32|ia64|both]).\n"
+		"--no-qual-check\t\t\t\tDo not check qualifier constraints on\n"
+		"\t\t\t\t\t events.\n"
+		"--insecure\t\t\t\tAllow rum/sum in monitored task\n"
+		"\t\t\t\t\t (per-thread mode and root only).\n"
+		"--excl-intr\t\t\t\tExclude interrupt-triggered execution\n"
+		"\t\t\t\t\t from system-wide measurement.\n"
+		"--intr-only\t\t\t\tInclude only interrupt-triggered\n"
+		"\t\t\t\t\t execution from system-wide measurement.\n"
 	);
 }
 
@@ -764,16 +772,18 @@ check_ita_event_combinations(pfmon_event_set_t *set)
 	unsigned int i, use_opcm, ev;
 	pfmlib_ita_input_param_t *param = set->mod_inp;
 	pfmon_ita_args_t *ita_args;
-	char name[PFMON_MAX_EVTNAME_LEN];
+	char *name;
 
 	ita_args = set->mod_args;
 
+	name = options.ev_name1;
+
 	use_opcm = param->pfp_ita_pmc8.opcm_used || param->pfp_ita_pmc9.opcm_used; 
 	for (i=0; i < set->event_count; i++) {
 
 		ev = set->inp.pfp_events[i].event;
 
-		pfm_get_event_name(ev, name, PFMON_MAX_EVTNAME_LEN);
+		pfm_get_event_name(ev, name, options.max_event_name_len+1);
 
 		if (use_opcm && pfm_ita_support_opcm(ev) == 0)
 			fatal_error("event %s does not support opcode matching\n", name);
@@ -787,6 +797,7 @@ check_ita_event_combinations(pfmon_event_set_t *set)
 		if (ita_args->opt_ia32 && ita_args->opt_ia64 == 0 && pfm_ita_is_btb(ev))
 			fatal_error("cannot use BTB event (%s) when only monitoring IA-32 execution\n", name);
 	}
+
 	/*
 	 * we do not call check_counter_conflict() because Itanium does not have events
 	 * which can only be measured on one counter, therefore this routine would not
@@ -963,18 +974,23 @@ pfmon_ita_print_header(FILE *fp)
 	pfmlib_ita_input_param_t *mod_in;
 	unsigned int i, k, l;
 	int isn;
-	char name[PFMON_MAX_EVTNAME_LEN];
+	char *name;
+	size_t len;
 	static const char *insn_str[]={
 		"ia32/ia64",
 		"ia32", 
 		"ia64"
 	};
+	len = 1 + options.max_event_name_len;
+	name = malloc(len);
+	if (!name)
+		fatal_error("cannot allocate string buffer");
 
 	for(k=0, set = options.sets; set; k++, set = set->next) {
 		mod_in = (pfmlib_ita_input_param_t *)set->mod_inp;
 		fprintf(fp, "#\n#\n# instruction sets for set%u:\n", k);
 		for(i=0; i < set->event_count; i++) {
-			pfm_get_event_name(set->inp.pfp_events[i].event, name, PFMON_MAX_EVTNAME_LEN);
+			pfm_get_event_name(set->inp.pfp_events[i].event, name, len);
 
 			isn =mod_in->pfp_ita_counters[i].ism;
 			fprintf(fp, "#\tPMD%d: %-*s = %s\n", 
@@ -986,11 +1002,12 @@ pfmon_ita_print_header(FILE *fp)
 		} 
 		fprintf(fp, "#\n");
 	}
+	free(name);
 	return 0;
 }
 
 static void
-pfmon_ita_detailed_event_name(int evt)
+pfmon_ita_detailed_event_name(unsigned int evt)
 {
 	unsigned long umask;
 	unsigned int maxincr;
@@ -1063,6 +1080,58 @@ pfmon_ita_verify_event_sets(void)
 	}
 }
 
+static void
+pfmon_ita_show_event_info(unsigned int idx)
+{
+	unsigned long umask;
+	unsigned int maxincr;
+	int n = 0;
+
+	pfm_ita_get_event_umask(idx, &umask);
+	pfm_ita_get_event_maxincr(idx, &maxincr);
+
+	printf("Umask    : 0x%lx\n", umask);
+
+	if (pfm_ita_is_dear(idx)) {
+		printf("EAR      : Data (%s)\n",
+			pfm_ita_is_dear_tlb(idx) ?  "TLB Mode": "Cache Mode");
+	} else if (pfm_ita_is_iear(idx)) {
+		printf("EAR      : Code (%s)\n",
+			pfm_ita_is_iear_tlb(idx) ?
+				"TLB Mode": "Cache Mode");
+	} else
+		puts("EAR      : None");
+
+	printf("BTB      : %s\n", pfm_ita_is_btb(idx) ? "Yes" : "No");
+
+	if (maxincr > 1)
+		printf("MaxIncr  : %u  (Threshold [0-%u])\n", maxincr,  maxincr-1);
+ 	else
+		printf("MaxIncr  : %u  (Threshold 0)\n", maxincr);
+
+	printf("Qual     : ");
+
+	if (pfm_ita_support_opcm(idx)) {
+		printf("[Opcode Match] ");
+		n++;
+	}
+
+	if (pfm_ita_support_iarr(idx)) {
+		printf("[Instruction Address Range] ");
+		n++;
+	}
+
+	if (pfm_ita_support_darr(idx)) {
+		printf("[Data Address Range] ");
+		n++;
+	}
+
+	if (n == 0)
+		puts("None");
+	else
+		putchar('\n');
+}
+
 pfmon_support_t pfmon_itanium = {
 	.name				= "Itanium",
 	.pmu_type			= PFMLIB_ITANIUM_PMU,
@@ -1080,4 +1149,5 @@ pfmon_support_t pfmon_itanium = {
 	.pfmon_setup_ctx_flags		= pfmon_ita_setup_ctx_flags,
 	.pfmon_verify_cmdline		= pfmon_ita_verify_cmdline,
 	.pfmon_verify_event_sets	= pfmon_ita_verify_event_sets,
+	.pfmon_show_event_info		= pfmon_ita_show_event_info
 };
