@@ -364,30 +364,39 @@ pfmon_ita2_install_pmd_registers(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 static void
 pfmon_ita2_usage(void)
 {
-	printf(
-		"--event-thresholds=thr1,thr2,...\tset event thresholds (no space)\n"
-		"--opc-match8=[mifb]:match:mask\t\tset opcode match for pmc8\n"
-		"--opc-match9=[mifb]:match:mask\t\tset opcode match for pmc9\n"
-		"--btb-tm-tk\t\t\t\tcapture taken IA-64 branches only\n"
-		"--btb-tm-ntk\t\t\t\tcapture not taken IA-64 branches only\n"
-		"--btb-ptm-correct\t\t\tcapture branch if target predicted correctly\n"
-		"--btb-ptm-incorrect\t\t\tcapture branch if target is mispredicted\n"
-		"--btb-ppm-correct\t\t\tcapture branch if path is predicted correctly\n"
-		"--btb-ppm-incorrect\t\t\tcapture branch if path is mispredicted\n"
-		"--btb-brt-iprel\t\t\t\tcapture IP-relative branches only\n"
-		"--btb-brt-ret\t\t\t\tcapture return branches only\n"
-		"--btb-brt-ind\t\t\t\tcapture non-return indirect branches only\n"
-		"--irange=start-end\t\t\tspecify an instruction address range constraint\n"
-		"--drange=start-end\t\t\tspecify a data address range constraint\n"
-		"--checkpoint-func=addr\t\t\ta bundle address to use as checkpoint\n"
-		"--ia32\t\t\t\t\tmonitor IA-32 execution only\n"
-		"--ia64\t\t\t\t\tmonitor IA-64 execution only\n"
-		"--insn-sets=set1,set2,...\t\tset per event instruction set (setX=[ia32|ia64|both])\n"
-		"--inverse-irange\t\t\tinverse instruction range restriction\n"
-		"--no-qual-check\t\t\t\tdo not check qualifier constraints on events\n"
-		"--insecure\t\t\t\tallow rum/sum in monitored task(per-thread mode only)\n"
-		"--excl-intr\t\t\t\texclude interrupt-triggered execution from system-wide measurement\n"
-		"--intr-only\t\t\t\tinclude only interrupt-triggered execution from system-wide measurement\n"
+	printf( "--event-thresholds=thr1,thr2,...\tSet event thresholds (no space).\n"
+		"--opc-match8=[mifb]:match:mask\t\tSet opcode match for pmc8.\n"
+		"--opc-match9=[mifb]:match:mask\t\tSet opcode match for pmc9.\n"
+		"--btb-tm-tk\t\t\t\tCapture taken IA-64 branches only.\n"
+		"--btb-tm-ntk\t\t\t\tCapture not taken IA-64 branches only.\n"
+		"--btb-ptm-correct\t\t\tCapture branch if target predicted\n"
+		"\t\t\t\t\t correctly.\n"
+		"--btb-ptm-incorrect\t\t\tCapture branch if target is\n"
+		"\t\t\t\t\t mispredicted.\n"
+		"--btb-ppm-correct\t\t\tCapture branch if path is predicted\n"
+		"\t\t\t\t\t correctly.\n"
+		"--btb-ppm-incorrect\t\t\tCapture branch if path is mispredicted.\n"
+		"--btb-brt-iprel\t\t\t\tCapture IP-relative branches only.\n"
+		"--btb-brt-ret\t\t\t\tCapture return branches only.\n"
+		"--btb-brt-ind\t\t\t\tCapture non-return indirect branches\n"
+		"\t\t\t\t\t only.\n"
+		"--irange=start-end\t\t\tSpecify an instruction address range\n"
+		"\t\t\t\t\t constraint.\n"
+		"--drange=start-end\t\t\tSpecify a data address range constraint.\n"
+		"--checkpoint-func=addr\t\t\tA bundle address to use as checkpoint.\n"
+		"--ia32\t\t\t\t\tMonitor IA-32 execution only.\n"
+		"--ia64\t\t\t\t\tMonitor IA-64 execution only.\n"
+		"--insn-sets=set1,set2,...\t\tSet per event instruction set\n"
+		"\t\t\t\t\t (setX=[ia32|ia64|both]).\n"
+		"--inverse-irange\t\t\tInverse instruction range restriction.\n"
+		"--no-qual-check\t\t\t\tDo not check qualifier constraints on\n"
+		"\t\t\t\t\t events.\n"
+		"--insecure\t\t\t\tAllow rum/sum in monitored task\n"
+		"\t\t\t\t\t (per-thread mode only).\n"
+		"--excl-intr\t\t\t\tExclude interrupt-triggered execution\n"
+		"\t\t\t\t\t from system-wide measurement.\n"
+		"--intr-only\t\t\t\tInclude only interrupt-triggered\n"
+		"\t\t\t\t\t execution from system-wide measurement.\n"
 	);
 }
 
@@ -935,7 +944,7 @@ check_cancel_events(pfmon_event_set_t *set)
 	char name[PFMON_MAX_EVTNAME_LEN], name2[PFMON_MAX_EVTNAME_LEN];
 
 	for(i=0; i < NCANCEL_EVENTS; i++) {
-		pfm_find_event_byname(cancel_events[i], &tmp);
+		pfm_find_event(cancel_events[i], &tmp);
 		pfm_get_event_code(tmp, &code);
 		cancel_codes[i] = code;
 	}
@@ -1052,7 +1061,7 @@ check_ita2_event_combinations(pfmon_event_set_t *set)
 
 	use_opcm = param->pfp_ita2_pmc8.opcm_used || param->pfp_ita2_pmc9.opcm_used; 
 
-	pfm_find_event_byname("IA64_INST_RETIRED", &inst_retired_idx);
+	pfm_find_event("IA64_INST_RETIRED", &inst_retired_idx);
 	pfm_get_event_code(inst_retired_idx, &inst_retired_code);
 
 	for (i=0; i < set->event_count; i++) {
@@ -1204,19 +1213,24 @@ pfmon_ita2_print_header(FILE *fp)
 	pfmlib_ita2_input_param_t *mod_in;
 	unsigned int i,k;
 	int isn;
-	char name[PFMON_MAX_EVTNAME_LEN];
+	size_t l;
+	char *name;
 	static const char *insn_str[]={
 		"ia32/ia64",
 		"ia32", 
 		"ia64"
 	};
+	l = 1 + options.max_event_name_len;
+	name = malloc(l);
+	if (!name)
+		fatal_error("cannot allocate string buffer");
 
 	for(k=0, set = options.sets; set; k++, set = set->next) {
 		mod_in = (pfmlib_ita2_input_param_t *)set->mod_inp;
 		fprintf(fp, "#\n#\n# instruction sets for set%u:\n", k);
 		for(i=0; i < set->event_count; i++) {
 
-			pfm_get_event_name(set->inp.pfp_events[i].event, name, PFMON_MAX_EVTNAME_LEN);
+			pfm_get_event_name(set->inp.pfp_events[i].event, name, l);
 
 			isn =mod_in->pfp_ita2_counters[i].ism;
 			fprintf(fp, "#\tPMD%d: %-*s = %s\n", 
@@ -1227,11 +1241,12 @@ pfmon_ita2_print_header(FILE *fp)
 		} 
 		fprintf(fp, "#\n");
 	}
+	free(name);
 	return 0;
 }
 
 static void
-pfmon_ita2_detailed_event_name(int evt)
+pfmon_ita2_detailed_event_name(unsigned int evt)
 {
 	unsigned long umask;
 	unsigned int maxincr;
@@ -1292,6 +1307,80 @@ pfmon_ita2_verify_cmdline(int argc, char **argv)
 	}
 }
 
+static void
+pfmon_ita2_show_event_info(unsigned int idx)
+{
+	unsigned long umask;
+	unsigned int maxincr;
+	int grp, set, n = 0;
+
+	pfm_ita2_get_event_umask(idx, &umask);
+	pfm_ita2_get_event_maxincr(idx, &maxincr);
+
+	printf("Umask    : 0x%lx\n", umask);
+
+	if (pfm_ita2_is_dear(idx)) {
+		printf("EAR      : Data (%s)\n",
+			pfm_ita2_is_dear_tlb(idx) ?
+				"TLB Mode": (pfm_ita2_is_dear_alat(idx) ? "ALAT Mode": "Cache Mode"));
+	} else if (pfm_ita2_is_iear(idx)) {
+		printf("EAR      : Code (%s)\n",
+			pfm_ita2_is_iear_tlb(idx) ?
+				"TLB Mode": "Cache Mode");
+	} else
+		puts("EAR      : None");
+
+	printf("BTB      : %s\n", pfm_ita2_is_btb(idx) ? "Yes" : "No");
+
+	if (maxincr > 1)
+		printf("MaxIncr  : %u  (Threshold [0-%u])\n", maxincr,  maxincr-1);
+ 	else
+		printf("MaxIncr  : %u  (Threshold 0)\n", maxincr);
+
+	printf("Qual     : ");
+
+	if (pfm_ita2_support_opcm(idx)) {
+		printf("[Opcode Match] ");
+		n++;
+	}
+
+	if (pfm_ita2_support_iarr(idx)) {
+		printf("[Instruction Address Range] ");
+		n++;
+	}
+
+	if (pfm_ita2_support_darr(idx)) {
+		printf("[Data Address Range] ");
+		n++;
+	}
+
+	if (n == 0)
+		puts("None");
+	else
+		putchar('\n');
+
+	pfm_ita2_get_event_group(idx, &grp);
+	pfm_ita2_get_event_set(idx, &set);
+
+	switch(grp) {
+		case PFMLIB_ITA2_EVT_NO_GRP:
+			puts("Group    : None");
+			break;
+		case PFMLIB_ITA2_EVT_L1_CACHE_GRP:
+			puts("Group    : L1D Cache");
+			break;
+		case PFMLIB_ITA2_EVT_L2_CACHE_GRP:
+			puts("Group    : L2 Cache");
+			break;
+		default:
+			puts("unknown");
+	}
+	if (set == PFMLIB_ITA2_EVT_NO_SET)
+		puts("Set      : None");
+	else
+		printf("Set      : %d\n", set);
+}
+
 pfmon_support_t pfmon_itanium2={
 	.name				="Itanium2",
 	.pmu_type			= PFMLIB_ITANIUM2_PMU,
@@ -1308,5 +1397,6 @@ pfmon_support_t pfmon_itanium2={
 	.pfmon_detailed_event_name	= pfmon_ita2_detailed_event_name,
 	.pfmon_setup_ctx_flags		= pfmon_ita2_setup_ctx_flags,
 	.pfmon_verify_event_sets	= pfmon_ita2_verify_event_sets,
-	.pfmon_verify_cmdline		= pfmon_ita2_verify_cmdline
+	.pfmon_verify_cmdline		= pfmon_ita2_verify_cmdline,
+	.pfmon_show_event_info		= pfmon_ita2_show_event_info
 };
