@@ -367,28 +367,40 @@ pfmon_mont_install_pmd_registers(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 static void
 pfmon_mont_usage(void)
 {
-	printf( "--event-thresholds=thr1,thr2,...\tset event thresholds (no space)\n"
-		"--opc-match32=mifb:match:mask\t\tset opcode match for pmc32\n"
-		"--opc-match34=mifb:match:mask\t\tset opcode match for pmc34\n"
-		"--etb-tm-tk\t\t\t\tcapture taken IA-64 branches only\n"
-		"--etb-tm-ntk\t\t\t\tcapture not taken IA-64 branches only\n"
-		"--etb-ptm-correct\t\t\tcapture branch if target predicted correctly\n"
-		"--etb-ptm-incorrect\t\t\tcapture branch if target is mispredicted\n"
-		"--etb-ppm-correct\t\t\tcapture branch if path is predicted correctly\n"
-		"--etb-ppm-incorrect\t\t\tcapture branch if path is mispredicted\n"
-		"--etb-brt-iprel\t\t\t\tcapture IP-relative branches only\n"
-		"--etb-brt-ret\t\t\t\tcapture return branches only\n"
-		"--etb-brt-ind\t\t\t\tcapture non-return indirect branches only\n"
-		"--irange=start-end\t\t\tspecify an instruction address range constraint\n"
-		"--drange=start-end\t\t\tspecify a data address range constraint\n"
-		"--checkpoint-func=addr\t\t\ta bundle address to use as checkpoint\n"
-		"--inverse-irange\t\t\tinverse instruction range restriction\n"
-		"--no-qual-check\t\t\t\tdo not check qualifier constraints on events\n"
-		"--insecure\t\t\t\tallow rum/sum in monitored task(per-thread mode only)\n"
-		"--excl-intr\t\t\texclude interrupt-triggered execution from system-wide measurement\n"
-		"--intr-only\t\t\tinclude only interrupt-triggered execution from system-wide measurement\n"
-		"--irange-demand-fetch\t\t\tlimit irange to demand fetched cache lines for specific prefetch events\n"
-		"--irange-prefetch-match\t\t\tlimit irange to explicitly prefetched cache lines for specific prefetch events\n"
+	printf( "--event-thresholds=thr1,thr2,...\tSet event thresholds (no space).\n"
+		"--opc-match32=mifb:match:mask\t\tSet opcode match for pmc32.\n"
+		"--opc-match34=mifb:match:mask\t\tSet opcode match for pmc34.\n"
+		"--etb-tm-tk\t\t\t\tCapture taken IA-64 branches only.\n"
+		"--etb-tm-ntk\t\t\t\tCapture not taken IA-64 branches only.\n"
+		"--etb-ptm-correct\t\t\tCapture branch if target predicted\n"
+		"\t\t\t\t\t correctly.\n"
+		"--etb-ptm-incorrect\t\t\tCapture branch if target is\n"
+		"\t\t\t\t\t mispredicted.\n"
+		"--etb-ppm-correct\t\t\tCapture branch if path is predicted\n"
+		"\t\t\t\t\t correctly.\n"
+		"--etb-ppm-incorrect\t\t\tCapture branch if path is mispredicted.\n"
+		"--etb-brt-iprel\t\t\t\tCapture IP-relative branches only.\n"
+		"--etb-brt-ret\t\t\t\tCapture return branches only.\n"
+		"--etb-brt-ind\t\t\t\tCapture non-return indirect branches\n"
+		"\t\t\t\t\t only.\n"
+		"--irange=start-end\t\t\tSpecify an instruction address range\n"
+		"\t\t\t\t\t constraint.\n"
+		"--drange=start-end\t\t\tSpecify a data address range constraint.\n"
+		"--checkpoint-func=addr\t\t\tA bundle address to use as checkpoint.\n"
+		"--inverse-irange\t\t\tInverse instruction range restriction.\n"
+		"--no-qual-check\t\t\t\tDo not check qualifier constraints on\n"
+		"\t\t\t\t\t events.\n"
+		"--insecure\t\t\t\tAllow rum/sum in monitored task\n"
+		"\t\t\t\t\t (per-thread mode only).\n"
+		"--excl-intr\t\t\t\tExclude interrupt-triggered execution\n"
+		"\t\t\t\t\t from system-wide measurement.\n"
+		"--intr-only\t\t\t\tInclude only interrupt-triggered\n"
+		"\t\t\t\t\t execution from system-wide measurement.\n"
+		"--irange-demand-fetch\t\t\tLimit irange to demand fetched cache\n"
+		"\t\t\t\t\t lines for specific prefetch events.\n"
+		"--irange-prefetch-match\t\t\tLimit irange to explicitly prefetched\n"
+		"\t\t\t\t\t cache lines for specific prefetch\n"
+		"\t\t\t\t\t events.\n"
 	);
 }
 
@@ -496,7 +508,6 @@ pfmon_mont_setup_etb(pfmon_event_set_t *set)
 	 * 	  This simplifies the number of cases quite substantially.
 	 * 	- change the data selector
 	 */
-	param->pfp_mont_etb.etb_ds  = 0;
 	param->pfp_mont_etb.etb_tm  = 0x3;
 	param->pfp_mont_etb.etb_ptm = 0x3;
 	param->pfp_mont_etb.etb_ppm = 0x3;
@@ -508,8 +519,7 @@ pfmon_mont_setup_etb(pfmon_event_set_t *set)
 	if (mont_args->opt_etb_ppm) param->pfp_mont_etb.etb_ppm = mont_args->opt_etb_ppm & 0x3;
 	if (mont_args->opt_etb_brt) param->pfp_mont_etb.etb_brt = mont_args->opt_etb_brt & 0x3;
 
-	vbprintf("etb options: ds=%d tm=%d ptm=%d ppm=%d brt=%d\n",
-		param->pfp_mont_etb.etb_ds,
+	vbprintf("etb options: ds=0 tm=%d ptm=%d ppm=%d brt=%d\n",
 		param->pfp_mont_etb.etb_tm,
 		param->pfp_mont_etb.etb_ptm,
 		param->pfp_mont_etb.etb_ppm,
@@ -862,10 +872,13 @@ check_cancel_events(pfmon_event_set_t *set)
 	int code, seen_first = 0;
 	int cancel_codes[NCANCEL_EVENTS];
 	unsigned int idx = 0;
-	char name[PFMON_MAX_EVTNAME_LEN], name2[PFMON_MAX_EVTNAME_LEN];
+	char *name1, *name2;
+
+	name1 = options.ev_name1;
+	name2 = options.ev_name2;
 
 	for(i=0; i < NCANCEL_EVENTS; i++) {
-		pfm_find_event_byname(cancel_events[i], &tmp);
+		pfm_find_event(cancel_events[i], &tmp);
 		pfm_get_event_code(tmp, &code);
 		cancel_codes[i] = code;
 	}
@@ -874,9 +887,9 @@ check_cancel_events(pfmon_event_set_t *set)
 			pfm_get_event_code(set->inp.pfp_events[i].event, &code);
 			if (code == cancel_codes[j]) {
 				if (seen_first) {
-					pfm_get_event_name(idx, name, PFMON_MAX_EVTNAME_LEN);
-					pfm_get_event_name(set->inp.pfp_events[i].event, name2, PFMON_MAX_EVTNAME_LEN);
-					fatal_error("%s and %s cannot be measured at the same time\n", name, name2);
+					pfm_get_event_name(idx, name1, options.max_event_name_len+1);
+					pfm_get_event_name(set->inp.pfp_events[i].event, name2, options.max_event_name_len+1);
+					fatal_error("%s and %s cannot be measured at the same time\n", name1, name2);
 				}
 				idx = set->inp.pfp_events[i].event;
 				seen_first = 1;
@@ -893,7 +906,6 @@ check_cross_groups(pfmon_event_set_t *set)
 	pfmlib_event_t *e= set->inp.pfp_events;
 	unsigned int cnt = set->event_count;
 	unsigned int i, j;
-	char name1[PFMON_MAX_EVTNAME_LEN], name2[PFMON_MAX_EVTNAME_LEN];
 
 	/*
 	 * Let check the L1D constraint first
@@ -948,9 +960,9 @@ check_cross_groups(pfmon_event_set_t *set)
 	}
 	return;
 error:
-	pfm_get_event_name(e[i].event, name1, PFMON_MAX_EVTNAME_LEN);
-	pfm_get_event_name(e[j].event, name2, PFMON_MAX_EVTNAME_LEN);
-	fatal_error("event %s and %s cannot be measured at the same time\n", name1, name2);
+	pfm_get_event_name(e[i].event, options.ev_name1, options.max_event_name_len+1);
+	pfm_get_event_name(e[j].event, options.ev_name2, options.max_event_name_len+1);
+	fatal_error("event %s and %s cannot be measured at the same time\n", options.ev_name1, options.ev_name2);
 error2:
 	fatal_error("you are using more L2D events from more than two sets at the same time\n");
 }
@@ -963,7 +975,6 @@ check_counter_conflict(pfmon_event_set_t *set)
 	unsigned int weight;
 	unsigned int cnt = set->event_count;
 	unsigned int i, j;
-	char name1[PFMON_MAX_EVTNAME_LEN], name2[PFMON_MAX_EVTNAME_LEN];
 
 	pfm_get_impl_counters(&all_counters);
 
@@ -979,9 +990,12 @@ check_counter_conflict(pfmon_event_set_t *set)
 	}
 	return;
 error:
-	pfm_get_event_name(e[i].event, name1, PFMON_MAX_EVTNAME_LEN);
-	pfm_get_event_name(e[j].event, name2, PFMON_MAX_EVTNAME_LEN);
-	fatal_error("event %s and %s cannot be measured at the same time, trying using different event sets\n", name1, name2);
+	pfm_get_event_name(e[i].event, options.ev_name1, options.max_event_name_len+1);
+	pfm_get_event_name(e[j].event, options.ev_name2, options.max_event_name_len+1);
+	fatal_error("event %s and %s cannot be measured at the same time, trying using"
+		    " different event sets\n",
+		    options.ev_name1,
+		    options.ev_name2);
 }
 
 static void
@@ -991,7 +1005,7 @@ check_mont_event_combinations(pfmon_event_set_t *set)
 	int code, inst_retired_code;
 	pfmlib_mont_input_param_t *param = set->mod_inp;
 	pfmon_mont_args_t *mont_args;
-	char name[PFMON_MAX_EVTNAME_LEN];
+	char *name;
 
 	mont_args = set->mod_args;
 
@@ -1008,14 +1022,16 @@ check_mont_event_combinations(pfmon_event_set_t *set)
 
 	use_opcm = param->pfp_mont_opcm1.opcm_used || param->pfp_mont_opcm2.opcm_used; 
 
-	pfm_find_event_byname("IA64_INST_RETIRED", &inst_retired_idx);
+	pfm_find_event("IA64_INST_RETIRED", &inst_retired_idx);
 	pfm_get_event_code(inst_retired_idx, &inst_retired_code);
 
+	name = options.ev_name1;
+
 	for (i=0; i < set->event_count; i++) {
 
 		ev = set->inp.pfp_events[i].event;
 
-		pfm_get_event_name(ev, name, PFMON_MAX_EVTNAME_LEN);
+		pfm_get_event_name(ev, name, options.max_event_name_len+1);
 		pfm_get_event_code(ev, &code);
 
 		if (use_opcm && pfm_mont_support_opcm(ev) == 0)
@@ -1114,13 +1130,12 @@ pfmon_mont_print_header(FILE *fp)
 }
 
 static void
-pfmon_mont_detailed_event_name(int evt)
+pfmon_mont_detailed_event_name(unsigned int evt)
 {
 	unsigned long umask;
 	unsigned int maxincr;
 	char *grp_str;
-	int grp, set;
-	int type;
+	int grp, set, type;
 
 	pfm_mont_get_event_umask(evt, &umask);
 	pfm_mont_get_event_maxincr(evt, &maxincr);
@@ -1128,7 +1143,10 @@ pfmon_mont_detailed_event_name(int evt)
 	pfm_mont_get_event_type(evt, &type);
 
 	printf("type=%c umask=0x%02lx incr=%u iarr=%c darr=%c opcm=%c ", 
-		type == PFMLIB_MONT_EVT_ACTIVE ? 'A' : (type == PFMLIB_MONT_EVT_FLOATING ? 'F' : 'C'),
+		type == PFMLIB_MONT_EVT_ACTIVE ?
+			'A' :
+			(type == PFMLIB_MONT_EVT_FLOATING ? 'F' :
+			 type == PFMLIB_MONT_EVT_CAUSAL ? 'C' : 'S'),
 		umask, 
 		maxincr,
 		pfm_mont_support_iarr(evt) ? 'Y' : 'N',
@@ -1179,8 +1197,100 @@ pfmon_mont_verify_cmdline(int argc, char **argv)
 	}
 }
 
+static void
+pfmon_mont_show_event_info(unsigned int idx)
+{
+	unsigned long umask;
+	unsigned int maxincr;
+	int grp, set, n = 0, type;
+	char *str;
+
+	pfm_mont_get_event_umask(idx, &umask);
+	pfm_mont_get_event_maxincr(idx, &maxincr);
+
+	printf("Umask    : 0x%lx\n", umask);
+
+	if (pfm_mont_is_dear(idx)) {
+		printf("EAR      : Data (%s)\n",
+			pfm_mont_is_dear_tlb(idx) ?
+				"TLB Mode": (pfm_mont_is_dear_alat(idx) ? "ALAT Mode": "Cache Mode"));
+	} else if (pfm_mont_is_iear(idx)) {
+		printf("EAR      : Code (%s)\n",
+			pfm_mont_is_iear_tlb(idx) ?
+				"TLB Mode": "Cache Mode");
+	} else
+		puts("EAR      : None");
+
+	printf("ETB      : %s\n", pfm_mont_is_etb(idx) ? "Yes" : "No");
+
+	if (maxincr > 1)
+		printf("MaxIncr  : %u  (Threshold [0-%u])\n", maxincr,  maxincr-1);
+ 	else
+		printf("MaxIncr  : %u  (Threshold 0)\n", maxincr);
+
+	printf("Qual     : ");
+
+	if (pfm_mont_support_opcm(idx)) {
+		printf("[Opcode Match] ");
+		n++;
+	}
+
+	if (pfm_mont_support_iarr(idx)) {
+		printf("[Instruction Address Range] ");
+		n++;
+	}
+
+	if (pfm_mont_support_darr(idx)) {
+		printf("[Data Address Range] ");
+		n++;
+	}
+
+	if (n == 0)
+		puts("None");
+	else
+		putchar('\n');
+
+	pfm_mont_get_event_type(idx, &type);
+	switch(type) {
+		case PFMLIB_MONT_EVT_ACTIVE:
+			str = "Active";
+			break;
+		case PFMLIB_MONT_EVT_FLOATING:
+			str = "Floating";
+			break;
+		case PFMLIB_MONT_EVT_CAUSAL:
+			str = "Causal";
+			break;
+		case PFMLIB_MONT_EVT_SELF_FLOATING:
+			str = "Self-Floating";
+			break;
+		default:
+			str = "??";
+	}
+	printf("Type     : %s\n", str);
+
+	pfm_mont_get_event_group(idx, &grp);
+	pfm_mont_get_event_set(idx, &set);
+
+	switch(grp) {
+		case PFMLIB_MONT_EVT_NO_GRP:
+			puts("Group    : None");
+			break;
+		case PFMLIB_MONT_EVT_L1D_CACHE_GRP:
+			puts("Group    : L1D Cache");
+			break;
+		case PFMLIB_MONT_EVT_L2D_CACHE_GRP:
+			puts("Group    : L2 Cache");
+			break;
+	}
+	if (set == PFMLIB_MONT_EVT_NO_SET)
+		puts("Set      : None");
+	else
+		printf("Set      : %d\n", set);
+}
+
 pfmon_support_t pfmon_montecito={
-	.name				="Dual-core Itanium 2",
+	.name				="dual-core Itanium 2",
 	.pmu_type			= PFMLIB_MONTECITO_PMU,
 	.generic_pmu_type		= PFMLIB_GEN_IA64_PMU,
 	.pfmon_initialize		= pfmon_mont_initialize,		
@@ -1195,5 +1305,6 @@ pfmon_support_t pfmon_montecito={
 	.pfmon_detailed_event_name	= pfmon_mont_detailed_event_name,
 	.pfmon_setup_ctx_flags		= pfmon_mont_setup_ctx_flags,
 	.pfmon_verify_event_sets	= pfmon_mont_verify_event_sets,
-	.pfmon_verify_cmdline		= pfmon_mont_verify_cmdline
+	.pfmon_verify_cmdline		= pfmon_mont_verify_cmdline,
+	.pfmon_show_event_info		= pfmon_mont_show_event_info
 };
