@@ -36,7 +36,7 @@ extern pfmon_smpl_module_t compact_ia64_old_smpl_module;   /* backward compatibi
 extern pfmon_smpl_module_t dear_hist_ia64_old_smpl_module;
 extern pfmon_smpl_module_t raw_ia64_old_smpl_module;
 
-extern pfmon_smpl_module_t detailed_x86_64_smpl_module;
+extern pfmon_smpl_module_t detailed_amd64_smpl_module;
 
 extern pfmon_smpl_module_t detailed_i386_p6_smpl_module;
 extern pfmon_smpl_module_t detailed_gen_ia32_smpl_module;
@@ -56,11 +56,13 @@ static pfmon_smpl_module_t *smpl_modules[]={
 	&raw_ia64_old_smpl_module,	/* for perfmon v2.0 */
 	&dear_hist_ia64_old_smpl_module,/* for perfmon v2.0 */
 #endif
-#ifdef CONFIG_PFMON_AMD_X86_64
-	&detailed_x86_64_smpl_module, /* must be first for X86-64 */
+#ifdef CONFIG_PFMON_AMD64
+	&detailed_amd64_smpl_module, /* must be first for X86-64 */
 #endif
 #ifdef CONFIG_PFMON_I386_P6
 	&detailed_i386_p6_smpl_module, /* must be first for P6 */
+#endif
+#ifdef CONFIG_PFMON_GEN_IA32
 	&detailed_gen_ia32_smpl_module, /* must be last for I386 */
 #endif
 	&inst_hist_smpl_module,		/* works for any PMU model */
