@@ -306,7 +306,7 @@ pfmon_sys_setup_context(pfmon_sdesc_t *sdesc, unsigned int cpu, pfmon_ctx_t *ctx
 
 	install_event_sets(sdesc);
 
-	if (pfmon_load_context(sdesc->ctxid, sdesc->tid) == -1) return -1;
+	if (pfmon_load_context(sdesc->ctxid, cpu) == -1) return -1;
 
 	return 0;
 }
@@ -483,7 +483,8 @@ do_measure_one_cpu(void *data)
 		}
 	}
 
-	if (pfmon_stop(ctxid) == -1) goto error;
+	if (pfmon_stop(ctxid) == -1)
+		warning("CPU%u could not stop monitoring, CPU may be offline, check results\n", mycpu);
 
 	vbprintf("CPU%-3u stopped monitoring\n", mycpu);
 
@@ -531,7 +532,8 @@ do_measure_one_cpu(void *data)
 	pthread_exit((void *)(0));
 	/* NO RETURN */
 error:
-	if (sdesc->ctxid > -1) close(sdesc->ctxid);
+	if (sdesc->ctxid > -1)
+		close(sdesc->ctxid);
 
 	vbprintf("CPU%-3u session aborted\n", mycpu);
 
