@@ -306,7 +306,9 @@ install_code_triggers(pfmon_sdesc_t *sdesc)
 		 */
 		trg->br_idx = i;
 		if (trg->trg_attr_start || trg->trg_attr_func == 0) {
-			ret = pfmon_set_code_breakpoint(pid, i, trg->brk_address);
+			ret = pfmon_set_code_breakpoint(pid, trg->br_idx,
+							trg->brk_address,
+							trg->trg_attr_start);
 			if (ret) {
 				warning("cannot install code breakpoints\n");
 				return -1;
@@ -335,7 +337,10 @@ install_data_triggers(pfmon_sdesc_t *sdesc)
 	for (i=0; i < num; i++, trg++) {
 		rw = trg->trg_attr_rw;
 		trg->br_idx = i;
-		ret = pfmon_set_data_breakpoint(pid, i, trg->brk_address, rw);
+		ret = pfmon_set_data_breakpoint(pid, i,
+						trg->brk_address,
+						rw,
+						trg->trg_attr_start);
 		if (ret) {
 			warning("cannot install data breakpoints\n");
 			return -1;
@@ -1244,13 +1249,13 @@ task_handle_trigger(pfmon_sdesc_t *sdesc)
 		stop_trg = sdesc->code_triggers+trg->stop_trg_idx;
 		stop_trg->brk_address = rp;
 
-		ret = pfmon_set_code_breakpoint(tid, stop_trg->br_idx, rp);
+		ret = pfmon_set_code_breakpoint(tid, stop_trg->br_idx, rp, 0);
 		if (ret) {
 			warning("cannot set dynamic stop breakpoint\n");
 			return -1;
 		}
 
-		vbprintf("[%d] install br.ret breakpoint @%p\n", tid, rp);
+		vbprintf("[%d] install return breakpoint @%p\n", tid, rp);
 	}
 skip_stop:
 	if (is_start) {
@@ -1263,9 +1268,13 @@ skip_stop:
 
 	if (is_repeat == 0) {
 		if (is_data)
-			pfmon_clear_data_breakpoint(tid, trg->br_idx, trg->brk_address);
+			pfmon_clear_data_breakpoint(tid, trg->br_idx,
+						    trg->brk_address,
+						    trg->trg_attr_start);
 		else
-			pfmon_clear_code_breakpoint(tid, trg->br_idx, trg->brk_address);
+			pfmon_clear_code_breakpoint(tid, trg->br_idx,
+						    trg->brk_address,
+						    trg->trg_attr_start);
 
 		vbprintf("[%d] clearing %s breakpoint @%p\n", 
 			tid, 
@@ -1274,9 +1283,15 @@ skip_stop:
 
 	} else  {
 		if (is_data)
-			pfmon_resume_after_data_breakpoint(tid);
+			pfmon_resume_after_data_breakpoint(tid,
+							   trg->br_idx,
+							   trg->brk_address,
+							   trg->trg_attr_start);
 		else
-			pfmon_resume_after_code_breakpoint(tid);
+			pfmon_resume_after_code_breakpoint(tid,
+							   trg->br_idx,
+							   trg->brk_address,
+							   trg->trg_attr_start);
 
 		vbprintf("[%d] resume after %s breakpoint\n", tid, is_data ? "data" : "code");
 	}
