@@ -104,36 +104,6 @@ not_found:
 	return ret;
 }
 
-#if 0
-static void
-extract_simple_numeric(char *file, char *field, unsigned long *num)
-{
-	FILE *fp;	
-	char *p;
-	unsigned long val = 0;
-	char buffer[64];
-
-	fp = fopen(file, "r");
-	if (fp == NULL) goto error;
-
-	for (;;) {
-		p  = fgets(buffer, sizeof(buffer)-1, fp);
-		if (p == NULL) break;
-
-		p = strchr(buffer, ':');
-		if (p == NULL) goto error;
-
-		*p = '\0'; 
-
-		if (!strncmp(field, buffer, strlen(field))) val = atoi(p+2);
-	}
-error:
-	if (fp) fclose(fp);
-
-	*num = val;
-}
-#endif
-
 void
 warning(char *fmt, ...)
 {
@@ -233,12 +203,9 @@ void
 setup_event_set(pfmon_event_set_t *set)
 {
 	char *event_name, *next_event_name;
-	char *mask_name = NULL; /* keep compiler happy */
-	char *next_mask_name;
 	size_t l;
-	unsigned int ev, um;
-	unsigned int cnt, num;
-	unsigned int num_masks;
+	unsigned int cnt;
+	int ret;
 
 	for (event_name = set->events_str, cnt = 0;
 	     event_name;
@@ -252,38 +219,10 @@ setup_event_set(pfmon_event_set_t *set)
 			*next_event_name++ = '\0';
 
 		l = strlen(event_name);
-
-		mask_name = strchr(event_name, ':');
-		if (mask_name)
-			*mask_name++ = '\0';
-		
-		if (pfm_find_event(event_name, &ev) != PFMLIB_SUCCESS)
+		ret = pfm_find_full_event(event_name, &set->inp.pfp_events[cnt]);
+		if (ret != PFMLIB_SUCCESS)
 			goto error;
 
-		set->inp.pfp_events[cnt].event = ev;
-
-		if (mask_name) {
-			pfm_get_num_event_masks(ev, &num_masks);
-			if (num_masks == 0)
-			goto no_umask;
-		}
-
-		for (num = 0; mask_name; mask_name = next_mask_name, num++) {
-			if (num == num_masks ||
-			    num == PFMLIB_MAX_MASKS_PER_EVENT)
-					goto many_umask;
-
-			next_mask_name = strchr(mask_name, ':');
-			if (next_mask_name)
-				*next_mask_name++ = '\0';
-
-			if (pfm_find_event_mask(ev, mask_name, &um) != PFMLIB_SUCCESS)
-				goto invalid_umask;
-
-			set->inp.pfp_events[cnt].unit_masks[num] = um;
-		}
-		set->inp.pfp_events[cnt].num_masks = num;
-
 		if (l > options.max_event_name_len)
 			options.max_event_name_len = l;
 	}
@@ -293,15 +232,9 @@ setup_event_set(pfmon_event_set_t *set)
 	return;
 
 error:
-	fatal_error("unknown event %s\n", event_name);
+	fatal_error("event %s : %s\n", event_name, pfm_strerror(ret));
 too_many:
 	fatal_error("too many events specified, max=%d\n", options.max_counters);
-no_umask:
-	fatal_error("event %s does not take any unit masks\n", event_name);
-invalid_umask:
-	fatal_error("unit mask %s does not exist\n", mask_name);
-many_umask:
-	fatal_error("too many umasks specified for event %s\n", event_name);
 }
 
 void
@@ -986,90 +919,6 @@ pfmon_clone_sets(pfmon_event_set_t *list, pfmon_sdesc_t *sdesc)
 	sdesc->nsets = cnt;
 }
 
-void
-pfmon_dump_sets(void)
-{
-	pfmon_event_set_t *set;
-	size_t max_len;
-	unsigned int i;
-	char name[PFMON_MAX_EVTNAME_LEN];
-	char event_str[PFMON_MAX_EVTNAME_LEN];
-
-	max_len = options.max_event_name_len;
-
-	printf("#include \"pfmon.h\"\n");
-
-	printf("static pfmon_event_set_t my_event_sets[]={\n");
-
-	/*
-	 * output set definitions
-	 */
-	for(set = options.sets; set; set=set->next) {
-		if (set->id) printf(",\n");
-		printf("/*set%-2u*/{\n\t\t.events_str =\"", set->id);
-		for(i=0; i < set->event_count; i++) {
-			pfm_get_event_name(set->inp.pfp_events[i].event, name, PFMON_MAX_EVTNAME_LEN);
-			if (i) putchar(',');
-			printf("%s", name);
-		}
-		if (set->mod_args) printf("\",\n\t\t.mod_args = &args_set%u\n", set->id);
-		printf("         }");
-	}
-	puts("\n};\n#define NR_EVENT_SETS sizeof(my_event_sets)/sizeof(pfmon_event_set_t)\n");
-
-	/*
-	 * output #define macros
-	 */
-	for(set = options.sets; set; set=set->next) {
-		for(i=0; i < set->event_count; i++) {
-			pfm_get_event_name(set->inp.pfp_events[i].event, name, PFMON_MAX_EVTNAME_LEN);
-			sprintf(event_str, "SET%u_%-*s", set->id, (int)max_len, name);
-			printf("#define %s\tsets[%u]->master_pd[%u].reg_value\n", 
-				event_str, 
-				set->id,
-				i);
-		}
-	}
-	puts("#define SMPL_MOD_NAME	\"my-measurement\"\n\n");
-	puts("\n\ntypedef struct {\n"
-	     "\tpfmon_event_set_t  *psets[NR_EVENT_SETS];\n"
-	     "} mdata_t;\n\n");
-
-	puts("/*\n"
-	     " * PFMON_RESULTS_FINAL : final call to the routine\n"
-	     " * PFMON_RESULTS_INTER : intermediate call to the routine\n"
-	     " */\n"
-	     "static int\n"
-	     "my_show_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode)\n"
-	     "{\n"
-	     "\tFILE *fp;\n"
-	     "\tpfmon_event_set_t **sets;\n"
-	     "\tmdata_t *mdata;\n\n"
-	     "\tfp    = sdesc->out_fp;\n"
-	     "\tmdata = (mdata_t *)sdesc->mdata;\n"
-	     "\tsets  = mdata->psets;\n"
-	     "\treturn 0;\n"
-	     "}\n");
-
-
-	puts("static int\n"
-	     "my_init_results(pfmon_sdesc_t *sdesc)\n"
-	     "{\n"
-	     "\tmdata_t *mdata;\n"
-	     "\tpfmon_event_set_t *pset;\n"
-	     "\tunsigned int i;\n\n"
-	     "\tsdesc->mdata = mdata = malloc(sizeof(mdata_t));\n"
-	     "\tif (mdata == NULL)\n"
-	     "\t\tfatal_error(\"cannot allocate private data\\n\");\n"
-	     "\tmemset(mdata, 0, sizeof(mdata_t));\n"
-	     "\tfor(pset = sdesc->sets, i=0; pset; i++,pset = pset->next) {\n"
-	     "\t\tmdata->psets[i] = pset;\n"
-	     "\t}\n"
-	     "\treturn 0;\n"
-	     "}\n");
-	puts("\n};\n");
-}
-
 int
 pfmon_validate_buf_size(size_t size)
 {
@@ -1112,17 +961,18 @@ pfmon_get_version_legacy(void)
 	char *buf = NULL;
 	char *p, *s;
 	size_t len = 0;
-	int ret;
-
+	int ret = -1;
 
 	fp = fopen("/proc/perfmon", "r");
 	if (fp == NULL)
 		return -1;
 
 	ret = getline(&buf, &len, fp);
-	if (ret < 0) fatal_error("cannot extract perfmon version\n");
+	if (ret < 0)
+		goto invalid_file;
 
-	if (strncmp(buf, "perfmon version", 15)) goto invalid_format;
+	if (strncmp(buf, "perfmon version", 15))
+		goto invalid_format;
 
 	p = strchr(buf+15, ':');
 	if (p == NULL) goto invalid_format;
@@ -1134,16 +984,12 @@ pfmon_get_version_legacy(void)
 	s  = p+1;
 	options.pfm_version |= atoi(s);
 
-	free(buf);
-
-	fclose(fp);
-
-	return 0;
-
+	ret = 0;
 invalid_format:
-	fatal_error("cannot parse perfmon version number\n");
-	/* never reached */
-	return -1;
+	free(buf);
+invalid_file:
+	fclose(fp);
+	return ret;
 }
 
 void
@@ -1163,56 +1009,24 @@ pfmon_get_version(void)
 		fatal_error("host kernel does not have perfmon support\n");
 
 	ret = getline(&buf, &len, fp);
-	if (ret < 0) fatal_error("cannot extract perfmon version\n");
+	if (ret < 0)
+		fatal_error("cannot extract perfmon version\n");
 
 	p = strchr(buf, '.');
-	if (p == NULL) goto invalid_format;
+	if (p == NULL)
+		goto invalid_format;
 	*p = '\0';
 	options.pfm_version = atoi(buf) << 16;
 	p++;
 	options.pfm_version |= atoi(p);
 
 	free(buf);
-
 	fclose(fp);
-
 	return;
-
 invalid_format:
 	fatal_error("cannot parse perfmon version number\n");
 }
 
-/*
- * only to be used with version 2.2 or higher
- * return:
- *    0 : a PMU module is registered with perfmon
- *    otherwise, PMU model description is missing
- */
-int
-pfmon_check_kernel_pmu(void)
-{
-	FILE *fp;
-	char *buf = NULL;
-	size_t len = 0;
-	int ret;
-
-	fp = fopen("/sys/kernel/perfmon/pmu_model", "r");
-	if (fp == NULL)
-		return -1;
-
-	ret = getline(&buf, &len, fp);
-	if (ret < 0)
-		return -1;
-
-	ret = strcasecmp(buf, "None") == 0 ? -1 : 0;
-
-	free(buf);
-
-	fclose(fp);
-
-	return ret;
-}
-
 int
 pfmon_cpu_virt_to_phys(int virt_cpu)
 {
