@@ -28,12 +28,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <stdarg.h>
-#include <pthread.h>
 #include <errno.h>
 #include <unistd.h>
-#include <fcntl.h>
-#include <ctype.h>
+#include <syscall.h>
 #include <sys/ptrace.h>
 #include <asm/ptrace_offsets.h>
 
@@ -438,9 +435,6 @@ ia64_set_psr_bit(pid_t pid, int bit, int mode)
 pid_t
 gettid(void)
 {
-#ifndef __NR_gettid
-#define __NR_gettid 1105
-#endif
 	return (pid_t)syscall(__NR_gettid);
 }
 
@@ -481,7 +475,7 @@ typedef union {
  * visible.
  */
 int
-pfmon_set_code_breakpoint(pid_t pid, int dbreg, uintptr_t address)
+pfmon_set_code_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start)
 {
 	dbreg_mask_reg_t mask;
 	int real_dbreg;
@@ -510,7 +504,7 @@ pfmon_set_code_breakpoint(pid_t pid, int dbreg, uintptr_t address)
 }
 
 int
-pfmon_clear_code_breakpoint(pid_t pid, int dbreg, uintptr_t address)
+pfmon_clear_code_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start)
 {
 	dbreg_mask_reg_t mask;
 	int real_dbreg;
@@ -548,7 +542,7 @@ pfmon_clear_code_breakpoint(pid_t pid, int dbreg, uintptr_t address)
  * 	bit 1 = r : 1 means trigger on read access
  */
 int
-pfmon_set_data_breakpoint(pid_t pid, int dbreg, uintptr_t address, int rw)
+pfmon_set_data_breakpoint(pid_t pid, int dbreg, unsigned long address, int rw, int is_start)
 {
 	dbreg_mask_reg_t mask;
 	long r;
@@ -578,7 +572,7 @@ pfmon_set_data_breakpoint(pid_t pid, int dbreg, uintptr_t address, int rw)
 }
 
 int
-pfmon_clear_data_breakpoint(pid_t pid, int dbreg, uintptr_t address)
+pfmon_clear_data_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start)
 {
 	dbreg_mask_reg_t mask;
 	int real_dbreg;
@@ -604,13 +598,13 @@ pfmon_clear_data_breakpoint(pid_t pid, int dbreg, uintptr_t address)
 }
 
 int
-pfmon_resume_after_code_breakpoint(pid_t pid)
+pfmon_resume_after_code_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start)
 {
 	return ia64_set_psr_bit(pid, PSR_ID_BIT, PSR_MODE_SET);
 }
 
 int
-pfmon_resume_after_data_breakpoint(pid_t pid)
+pfmon_resume_after_data_breakpoint(pid_t pid, int dbreg, unsigned long address, int is_start)
 {
 	return ia64_set_psr_bit(pid, PSR_DD_BIT, PSR_MODE_SET);
 }
@@ -794,18 +788,12 @@ pfmon_get_return_pointer(pid_t pid, uintptr_t *rp)
 int
 __pfmon_set_affinity(pid_t pid, size_t size, pfmon_cpumask_t mask)
 {
-#ifndef __NR_sched_setaffinity
-#define __NR_sched_setaffinity 1231
-#endif
 	return (int)syscall(__NR_sched_setaffinity, pid, size, mask);
 }
 
 int
 __pfmon_get_affinity(pid_t pid, size_t size, pfmon_cpumask_t mask)
 {
-#ifndef __NR_sched_getaffinity
-#define __NR_sched_getaffinity 1232
-#endif
 	return (int)syscall(__NR_sched_getaffinity, pid, size, mask);
 }
 
