@@ -302,15 +302,21 @@ static struct option dear_hist_cmd_options[]={
 static void
 dear_hist_show_options(void)
 {
-	printf("\t--smpl-show-top=n\t\tshow only the top n entries in the histogram (default: all entries)\n");
-	printf("\t--smpl-inst-view\t\tshow instruction address based histogram (default)\n");
-	printf("\t--smpl-data-view\t\tshow data address based histogram\n");
-	printf("\t--smpl-level-view\t\tshow cache/tlb level based histogram\n");
-	printf("\t--smpl-sort-bycount\t\tsort samples by number of count received (default)\n");
-	printf("\t--smpl-sort-byvalue\t\tsort samples by their value\n");
-	printf("\t--smpl-sort-bylevel\t\tsort samples by cache/tlb hit level\n");
-	printf("\t--smpl-save-raw\t\t\tsave samples in binary format for offline processing\n");
-	printf("\t--smpl-show-function\t\tshow per-function histograms (default per address)\n");
+	printf( "\t--smpl-show-top=n\t\tShow only the top n entries in the\n"
+		"\t\t\t\t\t histogram (default: all entries).\n"
+		"\t--smpl-inst-view\t\tShow instruction address based histogram\n"
+		"\t\t\t\t\t (default).\n"
+		"\t--smpl-data-view\t\tShow data address based histogram.\n"
+		"\t--smpl-level-view\t\tShow cache/tlb level based histogram.\n"
+		"\t--smpl-sort-bycount\t\tSort samples by number of count received\n"
+		"\t\t\t\t\t (default).\n"
+		"\t--smpl-sort-byvalue\t\tSort samples by their value.\n"
+		"\t--smpl-sort-bylevel\t\tSort samples by cache/tlb hit level.\n"
+		"\t--smpl-save-raw\t\t\tSave samples in binary format for\n"
+		"\t\t\t\t\t offline processing.\n"
+		"\t--smpl-show-function\t\tShow per-function histograms (default\n"
+		"\t\t\t\t\t per address).\n"
+	);
 }
 
 /*
